<?php

namespace app\admin\controller\setting;

use app\admin\model\box\Box;
use app\common\controller\Backend;
use think\Db;
use think\exception\PDOException;
use think\exception\ValidateException;

/**
 * 轮播图管理
 *
 * @icon fa fa-circle-o
 */
class Banner extends Backend
{

    protected $modelValidate = true;

    /**
     * Banner模型对象
     * @var \app\admin\model\setting\Banner
     */
    protected $model = null;

    public function _initialize()
    {
        parent::_initialize();
        $this->model = new \app\admin\model\setting\Banner;
        $this->view->assign("placeList", $this->model->getPlaceList());
        $this->view->assign("typeList", $this->model->getTypeList());
        $this->view->assign("statusList", $this->model->getStatusList());
    }

    public function import()
    {
        parent::import();
    }

    /**
     * 默认生成的控制器所继承的父类中有index/add/edit/del/multi五个基础方法、destroy/restore/recyclebin三个回收站方法
     * 因此在当前控制器中可不用编写增删改查的代码,除非需要自己控制这部分逻辑
     * 需要将application/admin/library/traits/Backend.php中对应的方法复制到当前控制器,然后进行修改
     */

    public function index()
    {
        //设置过滤方法
        $this->request->filter(['strip_tags', 'trim']);
        if ($this->request->isAjax()) {
            //如果发送的来源是Selectpage，则转发到Selectpage
            if ($this->request->request('keyField')) {
                return $this->selectpage();
            }
            list($where, $sort, $order, $offset, $limit) = $this->buildparams();

            // 默认查看首页轮播图
            $where_place = ['place' => 'index'];
            $filter = json_decode(input('filter'), true);
            if ($filter) {
                $where_place = [];
            }

            $list = $this->model
                ->field($this->indexField, $this->indexFieldExcept)
                ->where($where)
                ->where($where_place)
                ->order($sort, $order)
                ->paginate($limit)
                ->each(function ($item) {
                    $item->hidden(['value']);
                });

            $result = array("total" => $list->total(), "rows" => $list->items());

            return json($result);
        }
        $this->assignconfig('tab_default', '#tab-place-index');
        return $this->view->fetch();
    }

    public function add()
    {
        if ($this->request->isPost()) {
            $params = $this->request->post("row/a");
            if ($params) {
                $params = $this->preExcludeFields($params);

                if ($this->dataLimit && $this->dataLimitFieldAutoFill) {
                    $params[$this->dataLimitField] = $this->auth->id;
                }
                $result = false;
                Db::startTrans();
                try {
                    //是否采用模型验证
                    if ($this->modelValidate) {
                        $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                        $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.add' : $name) : $this->modelValidate;
                        $this->model->validateFailException(true)->validate($validate);
                    }
                    if ('normal' == $params['status']) {
                        switch ($params['type']) {
                            case 'box':
                                // 检查盲盒
                                if (empty($params['box']) || !is_numeric($params['box'])) {
                                    $this->error('盲盒选择有误');
                                }
                                $checkBox = Box::where('id', $params['box'])->value('id');
                                if (empty($checkBox)) {
                                    $this->error('盲盒不存在');
                                }
                                $params['value'] = $params['box'];
                                break;
                            case 'link':
                                if (empty($params['link']) || 0 !== strpos($params['link'], 'http')) {
                                    $this->error('链接设置有误');
                                }
                                $params['value'] = $params['link'];
                                break;
                            case 'word':
                                $params['value'] = $params['word'];
                        }
                    }
                    $result = $this->model->allowField(true)->save($params);
                    Db::commit();
                } catch (ValidateException $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                } catch (PDOException $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                } catch (\Exception $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                }
                if ($result !== false) {
                    $this->success();
                } else {
                    $this->error(__('No rows were inserted'));
                }
            }
            $this->error(__('Parameter %s can not be empty', ''));
        }
        return $this->view->fetch();
    }

    public function edit($ids = null)
    {
        $row = $this->model->get($ids);
        if (!$row) {
            $this->error(__('No Results were found'));
        }
        $adminIds = $this->getDataLimitAdminIds();
        if (is_array($adminIds)) {
            if (!in_array($row[$this->dataLimitField], $adminIds)) {
                $this->error(__('You have no permission'));
            }
        }
        if ($this->request->isPost()) {
            $params = $this->request->post("row/a");
            if ($params) {
                $params = $this->preExcludeFields($params);
                $result = false;
                Db::startTrans();
                try {
                    //是否采用模型验证
                    if ($this->modelValidate) {
                        $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                        $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.edit' : $name) : $this->modelValidate;
                        $row->validateFailException(true)->validate($validate);
                    }
                    if ('normal' == $params['status']) {
                        switch ($params['type']) {
                            case 'box':
                                // 检查盲盒
                                if (empty($params['box']) || !is_numeric($params['box'])) {
                                    $this->error('盲盒选择有误');
                                }
                                $checkBox = Box::where('id', $params['box'])->value('id');
                                if (empty($checkBox)) {
                                    $this->error('盲盒不存在');
                                }
                                $params['value'] = $params['box'];
                                break;
                            case 'link':
                                if (empty($params['link']) || 0 !== strpos($params['link'], 'http')) {
                                    $this->error('链接设置有误');
                                }
                                $params['value'] = $params['link'];
                                break;
                            case 'word':
                                $params['value'] = $params['word'];
                        }
                    }

                    $result = $row->allowField(true)->save($params);
                    Db::commit();
                } catch (ValidateException $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                } catch (PDOException $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                } catch (\Exception $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                }
                if ($result !== false) {
                    $this->success();
                } else {
                    $this->error(__('No rows were updated'));
                }
            }
            $this->error(__('Parameter %s can not be empty', ''));
        }
        $this->view->assign("row", $row);
        return $this->view->fetch();
    }

}
