<?php

namespace app\admin\controller\setting;

use app\common\controller\Backend;
use think\Db;
use think\exception\PDOException;
use think\exception\ValidateException;

/**
 * 平台设置管理
 *
 * @icon fa fa-circle-o
 */
class Setting extends Backend
{

    /**
     * Setting模型对象
     * @var \app\admin\model\setting\Setting
     */
    protected $model = null;

    public function _initialize()
    {
        parent::_initialize();
        $this->model = new \app\admin\model\setting\Setting;

    }

    /**
     * 默认生成的控制器所继承的父类中有index/add/edit/del/multi五个基础方法、destroy/restore/recyclebin三个回收站方法
     * 因此在当前控制器中可不用编写增删改查的代码,除非需要自己控制这部分逻辑
     * 需要将application/admin/library/traits/Backend.php中对应的方法复制到当前控制器,然后进行修改
     */

    public function index()
    {
        $row = $this->model->get(1);
      
        $adminIds = $this->getDataLimitAdminIds();
        if (is_array($adminIds)) {
            if (!in_array($row[$this->dataLimitField], $adminIds)) {
                $this->error(__('You have no permission'));
            }
        }
        if ($this->request->isPost()) {
            $params = $this->request->post("row/a");
            
           
            if ($params) {
                $params = $this->preExcludeFields($params);
                if (!is_numeric($params['one_rmb_to_coin_num']) || $params['one_rmb_to_coin_num'] < 0) {
                    $this->error('金币兑换RMB数量有误');
                }
                if (!is_numeric($params['recovery_discount']) || $params['recovery_discount'] < 0) {
                    $this->error('商品回收金额占比设置有误');
                }
                if (!is_numeric($params['retail_1']) || $params['retail_1'] < 0) {
                    $this->error('一级分销佣金占比设置有误');
                }
                if (!is_numeric($params['retail_1']) || $params['retail_1'] < 0) {
                    $this->error('二级分销佣金占比设置有误');
                }
                if (!is_numeric($params['retail_1']) || $params['retail_1'] < 0) {
                    $this->error('三级分销佣金占比设置有误');
                }
                $result = false;
                Db::startTrans();
                try {
                    //是否采用模型验证
                    if ($this->modelValidate) {
                        $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                        $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.edit' : $name) : $this->modelValidate;
                        $row->validateFailException(true)->validate($validate);
                    }
                  
                    $result = $row->allowField('logo_image,one_rmb_to_coin_num,hot_box_banner,cheap_box_banner,recovery_discount,service_number,service_qrcode,retail_1,retail_2,retail_3,kou,url,openurl,registerurl')->save($params);
                    Db::commit();
                } catch (ValidateException $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                } catch (PDOException $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                } catch (\Exception $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                }
                if ($result !== false) {
                    $this->success();
                } else {
                    $this->error(__('No rows were updated'));
                }
            }
            $this->error(__('Parameter %s can not be empty', ''));
        }
        $this->view->assign("row", $row);
        return $this->view->fetch();
    }

}
